TYPE WavHeader
  rID AS STRING * 4  ' "RIFF"
  rLen AS DWORD  ' length of data chunk
  wID AS STRING * 4  ' "WAVE"
  fID AS STRING * 4  ' "fmt"
  fLen AS DWORD
  wFormatTag AS WORD  ' 1 = pcm, ...
  nChannels AS WORD  ' 1=mono, 2 = stereo
  nSamplesPerSec AS WORD  ' playback frequency
  unknown1 AS WORD
  nAvgBytesPerSec AS WORD
  unknown2 AS WORD
  nBlockAlign AS WORD
  FormatSpecific AS WORD
  dID AS STRING * 4  ' "data"
  dLen AS DWORD  ' length of actual data
END TYPE

SHARED DSPReset%, DSPRead%, DSPWrite%, DSPAvailable%
%ADDRSBRESET = &H06
%ADDRSBREAD = &H0A
%ADDRSBWRITE = &H0C
%ADDRSBAVAILABLE = &H0E
%MAXBUFFER = &H2000  ' enough for most WAV files

SUB PlayWaveFile(DataFile$)
  IF BlasterHere% = 0 THEN EXIT SUB  ' no SB found
  BlasterAddress% = GetBlasterAddr%
  IF SBCardReset(BlasterAddress%) = 0 THEN EXIT SUB  ' could not reset SB
  IF LEN(DIR$(DataFile$)) = 0 THEN EXIT SUB  ' .WAV file not found
  ValidWavHeader DataFile$, LenHeader%, Channels%, Sampling%, ok%
  IF ok% = 0 THEN EXIT SUB  ' invalid .WAV file
  hnd% = FREEFILE
  OPEN "B", #hnd%, DataFile$
  Remaining& = LOF(hnd%) - LenHeader%
  SEEK #hnd%, LenHeader%
  magic??=2
  BufferLen% = %MAXBUFFER \ 6
  MTIMER
  FOR i% = 1 TO BufferLen%
    WriteByt? = PEEK(ptr%)
    FOR j% = 1 TO magic??
      WHILE((INP(DSPWrite%) AND &H80) <> 0) : WEND
      OUT DSPWrite%, &H10
      WHILE((INP(DSPWrite%) AND &H80) <> 0) : WEND
      OUT DSPWrite%, 0
    NEXT
    INCR ptr%, advance%
  NEXT
  Uhr& = MTIMER
  magic?? = BITS??(3200000000 \ (Sampling% * Uhr&))
  'magic?? = magic&&
  SBCardSpeakerOn
  DO WHILE Remaining& > 0
    IF Remaining& > %MAXBUFFER THEN
      BufferLen% = %MAXBUFFER
      Remaining& = Remaining& - %MAXBUFFER
    ELSE
      BufferLen% = Remaining&
      Remaining& = 0
    END IF
    Buffer$ = SPACE$(BufferLen%)
    GET$ #hnd%, BufferLen%, Buffer$
    DEF SEG = STRSEG(Buffer$)
    ptr% = STRPTR(Buffer$)
    IF Channels% = 1 OR Channels% = 2 THEN advance% = Channels%
    FOR i% = 1 TO BufferLen%
      WriteByt? = PEEK(ptr%)
      FOR j% = 1 TO magic??
        WHILE((INP(DSPWrite%) AND &H80) <> 0) : WEND
        OUT DSPWrite%, &H10
        WHILE((INP(DSPWrite%) AND &H80) <> 0) : WEND
        OUT DSPWrite%, WriteByt?
      NEXT
      INCR ptr%, advance%
    NEXT
  LOOP UNTIL LEN(INKEY$)
  DEF SEG
  SBCardSpeakerOff
  CLOSE #hnd%
END SUB

FUNCTION BlasterHere%
  BlasterHere% = (LEN(ENVIRON$("BLASTER")) > 0)
END FUNCTION

FUNCTION GetBlasterAddr%
  a$ = UCASE$(ENVIRON$("BLASTER"))
  p1% = INSTR(a$, "A")
  p2% = INSTR(p1% + 1, a$, " ")
  GetBlasterAddr% = VAL("&H" + MID$(a$, p2% - p1%))
END FUNCTION

FUNCTION SBCardReset(BYVAL addr%)
  DSPReset% = addr% + %ADDRSBRESET
  DSPRead% = addr% + %ADDRSBREAD
  DSPWrite% = addr% + %ADDRSBWRITE
  DSPAvailable% = addr% + %ADDRSBAVAILABLE
  OUT DSPReset%, 1
  DELAY 0.1  ' 3 micro seconds needed but longer just to be safe
  OUT DSPReset%, 0
  DELAY 0.1
  IF ((INP(DSPAvailable%) AND &H80) = &H80) THEN
    IF INP(DSPRead%) = &HAA THEN SBCardReset% = -1 ELSE SBCardReset% = 0
  ELSE
    SBCardReset% = 0
  END IF
END FUNCTION

SUB WriteToDSP(BYVAL v%)
  WHILE((INP(DSPWrite%) AND &b10000000) <> 0) : WEND
  OUT DSPWrite%, v%
END SUB

SUB WriteDAC(BYVAL v%)
  WriteToDSP &H10  ' direct mode to DAC
  WriteToDSP v%  ' 8-bit sample
END SUB

SUB SBCardSpeakerOn
  WriteToDSP &HD1
END SUB

SUB SBCardSpeakerOff
  WriteToDSP &HD3
END SUB

SUB ValidWavHeader(BYVAL f$, l%, c%, s%, ok%)
  DIM header AS WavHeader
  hnd% = FREEFILE
  OPEN "b", #hnd%, f$
  GET #hnd%,, header
  CLOSE #hnd%
  l% = LEN(header) + header.fLen - 16
  c% = header.nChannels
  s% = header.nSamplesPerSec
  IF UCASE$(header.rID) = "RIFF" AND UCASE$(header.wID) = "WAVE" AND UCASE$(header.fID) = "FMT " THEN ok% = -1 ELSE ok% = 0
END SUB
